#include <stdio.h>
#include <stdlib.h>
#include <cv.h>
#include <highgui.h>
#include <math.h>

void quantization(IplImage *img, IplImage *img_q, int nColor) {
  int height = img->height; // altura
  int width  = img->width;  // largura
  int step   = img->widthStep; // tamanho de uma linha
  uchar *data= img->imageData; // ponteiro para o array de pixels

  int step2   = img_q->widthStep; // tamanho de uma linha
  uchar *data2= img_q->imageData; // ponteiro para o array de pixels
  
  int i,j;

  for (i = 0; i < height; i++) {
      for (j = 0; j < width; j++) {
	   // acessa pixel de imagem com 3 canais de cor:
	   // posicao i,j
	   // canal k
	   // data[i*step+j*3+k] 	   
	   uchar C1 = (data[i*step+j*3+0] & 192)>>2; 
	   uchar C2 = (data[i*step+j*3+1] & 192)>>4; 
	   uchar C3 = (data[i*step+j*3+2] & 192)>>6;	   
	   data2[i*step2+j] = C1 | C2 | C3;
      }    
  } 
}

void GCH(IplImage *img, double *descriptor, int nColor) {
 
  IplImage *img_quant;
  // aloca uma nova imagem do tamanho da original
  // com 8 bits por pixel e 1 canal de cor
  img_quant = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1); 

  quantization(img, img_quant, nColor);
  
  int i,j;
  int height = img_quant->height; // altura
  int width  = img_quant->width;  // largura  
  int step   = img_quant->widthStep; // tamanho de uma linha
  uchar *data= img_quant->imageData; // ponteiro para o array de pixels
  
  // percorre cada pixel da imagem incrementando
  // o valor de cada cor no descritor
  for (i = 0; i < height; i++) {
      for (j = 0; j < width; j++) { 
	   descriptor [data[i*step+j]] ++;
      }    
  }
  // normaliza o descritor
  for (i = 0; i < nColor; i++) {
      descriptor[i] = descriptor[i]/ (double)(width*height);    
  }
  
}

// descriptor
// 0..(nColor-1) = Borda
// nColor..(nColor*2-1) = Interior
void BIC(IplImage *img, double *descriptor, int nColor) {
 
  IplImage *img_quant;
  // aloca uma nova imagem do tamanho da original
  // com 8 bits por pixel e 1 canal de cor
  img_quant = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1); 

  quantization(img, img_quant, nColor);
  
  int i,j;
  int height = img_quant->height; // altura
  int width  = img_quant->width;  // largura  
  int step   = img_quant->widthStep; // tamanho de uma linha
  uchar *data= img_quant->imageData; // ponteiro para o array de pixels
  
  
  // percorre cada pixel da imagem incrementando
  // o valor de cada cor no descritor
  for (i = 0; i < height; i++) {
      for (j = 0; j < width; j++) { 
	   // armazena pixel atual
	   uchar aux = data[i*step+j];
	   if (i > 0 && j > 0 && i == height && j == width) {
	      // se o pixel tiver cor igual a dos 4-vizinhos
	      if ((aux == data[(i-1)*step+j]) &&
		  (aux == data[(i+1)*step+j]) &&
		  (aux == data[i*step+(j-1)]) &&
		  (aux == data[i*step+(j+1)])) 
	      {
		  // incrementa histograma interior
		  descriptor[aux+nColor] ++;
	      } else {
		  // incrementa histograma borda
		  descriptor[aux] ++;
	      }
	   }
	   else {
	      // incrementa histograma borda
	      descriptor[aux] ++;
	   }
      }    
  }
  // normaliza o descritor
  for (i = 0; i < nColor; i++) {
      descriptor[i] = descriptor[i]/ (double)(width*height);    
  } 
}

void CoocurrenceMatrix(IplImage *img, double **Cm, int nColor, int dX, int dY) {
 int i,j;
 IplImage *img_quant;
 if (img->nChannels > 1) {
    img_quant = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1);
    quantization(img, img_quant, nColor);
 }
 else {
    img_quant = cvCloneImage(img);
 }
 
 // alocar uma nova imagem de tamanho maior para ser processada
 IplImage *img_b = cvCreateImage(cvSize(img_quant->width+((dY+1)*2), img_quant->height+((dX+1)*2) ), img_quant->depth, img_quant->nChannels);
 CvPoint offset = cvPoint(dX+1, dY+1);
 // copia a imagem quantizada e replica os pixels das bordas
 cvCopyMakeBorder(img_quant, img_b, offset, IPL_BORDER_REPLICATE, cvScalarAll(0));
 
 uchar *data = (uchar *)img_b->imageData;
 int step = img_b->widthStep;
 int height = img_b->height;
 int width = img_b->width;
 
 // percorre a imagem
 for (i = dX; i < height-dX; i++) {
      for (j = dY; j < width-dY; j++) {
	  int pref = data[i*step+j]; //pixel referencia
	  int pviz = data[(i+dX)*step+(j+dY)]; //pixel vizinho
	  Cm[pref][pviz]++;
      }
 }
 double sum = 0;
 // calcula a soma das ocorrencias
 for (i=0; i<nColor; i++) {
      for (j=0; j<nColor; j++) {
	    sum+= Cm[i][j];
      }
 }
 // normaliza a matriz de forma que sum(Cm) = 1
 for (i=0; i<nColor; i++) {
      for (j=0; j<nColor; j++) {
	    Cm[i][j] /= sum;
      }
 }
}

void Haralick(double **Cm, int size, double *HDesc) {
    
    int i,j;
    double m_r = 0.0;
    double m_c = 0.0;
    double s_r = 0.0;
    double s_c = 0.0;
    double *Pi = (double *) calloc(size, sizeof(double));
    double *Pj = (double *) calloc(size, sizeof(double));
  
    for (i = 0; i<size; i++) {
	for (j = 0; j<size; j++) {
	      Pi[i] += Cm[i][j];
	}
	m_r += i*Pi[i];	
    }
    for (j = 0; j<size; j++) {
	for (i = 0; i<size; i++) {
	      Pj[j] += Cm[i][j];
	}
	m_c += j*Pj[j];	
    }
    
    for (i = 0; i<size; i++) {
	s_r += ((i-m_r)*(i-m_r)) * Pi[i];
	s_c += ((i-m_c)*(i-m_c)) * Pj[i];
    }
    s_r = sqrt(s_r);
    s_c = sqrt(s_c);
    
    double maxp = 0.0;
    double corr = 0.0;
    double cont = 0.0;
    double unif = 0.0;
    double homo = 0.0;
    double entr = 0.0;
    double auxv = 0.0;
    
    for (i = 0; i<size; i++) {
      for (j = 0; j<size; j++) {
	  auxv = Cm[i][j];
	  // Descritor 1 - Maxima Probabilidade
	  if (maxp < auxv) {
	      maxp = auxv;
	  }
	  // Descritor 2 - correlacao
	  if (s_r > 0 && s_c > 0) {
	      corr+= ((i-m_r)*(j-m_c)*auxv) / (s_r*s_c);	    
	  }
	  // Descritor 3 - contraste
	  cont += ( (i-j)*(i-j)*auxv );
	  // Descritor 4 - energia (uniformidade)
	  unif += (auxv*auxv);
	  // Descritor 5 - homogeneidade 
	  unif += (auxv) / (1 + abs(i-j));
	  // Descritor 6 - entropia
	  if (auxv != 0) {
	    //entr+= auxv*( log(auxv) / log(2) );
	    entr+= auxv*( log2(auxv));
	  }
      }
    }
	
    entr = -entr;
    
    HDesc[0] = maxp;
    HDesc[1] = corr;
    HDesc[2] = cont;
    HDesc[3] = unif;
    HDesc[4] = homo;
    HDesc[5] = entr;  
}

double distManhattan(double *p, double *q, int size) {
    int i;
    double dist =0;
    for (i=0; i<size; i++) {
	  dist += fabs(p[i]-q[i]);      
    }
    return dist;  
}