#include <stdio.h>
#include <stdlib.h>
#include <cv.h>
#include <highgui.h>
#include <math.h>
#include "descriptors.h"

int main (int argc, char *argv[]) {
    
    IplImage *img1, *img2, *imgRg, *imgRb;
    int nColor = 64; // numero de cores para quantizar a imagem
    int i;
    // carega a imagem sem alterar o sistema de cores
    img1 = cvLoadImage(argv[1], CV_LOAD_IMAGE_UNCHANGED);
    
    double **COM1A = (double **)calloc(nColor, sizeof(double*));
    double **COM1B = (double **)calloc(nColor, sizeof(double*));
    for (i=0; i<nColor; i++) {
	  COM1A[i]= (double *)calloc(nColor, sizeof(double));
	  COM1B[i]= (double *)calloc(nColor, sizeof(double));
    }
    
    // vetor para armazenar os descritores
    double *GCH1 = (double *)calloc(nColor, sizeof(double));
    double *GCH2 = (double *)calloc(nColor, sizeof(double));
    // Haralick Descriptors
    double *HD1 = (double *)calloc(12, sizeof(double));
    double *HD2 = (double *)calloc(12, sizeof(double));
  
    // calcula o descritor da imagem de consulta
    GCH(img1, GCH1, nColor);
    
    // calcula matriz de coocorrencia e descritores de Haralick
    // da imagem de consulta
    // imagem, matriz, quantizacao, Q = (1,0) e Q = (0,1)
    CoocurrenceMatrix(img1, COM1A, nColor, 1, 0);
    CoocurrenceMatrix(img1, COM1B, nColor, 0, 1);
    Haralick(COM1A, nColor, HD1);
    Haralick(COM1B, nColor, &HD1[6]);
        
    double minDistGCH = DBL_MAX, minDistH = DBL_MAX;
    char * filename = (char*)calloc(128, sizeof(char));
    for (i=0; i<15; i++) {	  
	  sprintf(filename,"./imagedatabase_textures/%.2d.jpg",i);
	  // carrega a imagem do banco de imagens
	  img2 = cvLoadImage(filename, CV_LOAD_IMAGE_UNCHANGED);
	  if (!img2) {
		printf("Erro ao abrir imagem %s\n", filename);
		return -1;
	  }
	  
	  // calcula o descritor da imagem no banco de imagens
	  GCH(img2, GCH2, nColor);
	  
	  CoocurrenceMatrix(img2, COM1A, nColor, 1, 0);
	  CoocurrenceMatrix(img2, COM1B, nColor, 3, 5);
	  Haralick(COM1A, nColor, HD2);
	  Haralick(COM1B, nColor, &HD2[6]);
	  
	  // compara os descritores
	  // GCH - tamanho do descritor = ncores
	  double distGCH = distManhattan(GCH1, GCH2, nColor);
	  // Haralick - tamanho do descritor = 12 (2 Matrizes)
	  double distH = distManhattan(HD1, HD2, 12);
	  
	  // comparar os descritores
	  if (distGCH < minDistGCH) {
	      minDistGCH = distGCH;
	      imgRg = cvCloneImage(img2);
	  }
	  if (distH < minDistH) {
	      minDistH = distH;
	      imgRb = cvCloneImage(img2);
	  }
    }
  
    // abre uma janela para exibir imagem
    cvNamedWindow("Imagem_Consulta", CV_WINDOW_AUTOSIZE);
    cvShowImage("Imagem_Consulta", img1);
    cvNamedWindow("Resultado GCH", CV_WINDOW_AUTOSIZE);
    cvShowImage("Resultado GCH", imgRg);
    cvNamedWindow("Resultado Haralick", CV_WINDOW_AUTOSIZE);
    cvShowImage("Resultado Haralick", imgRb);
    cvWaitKey(0);
    return 0;
}
